<?php

namespace WSOL_WP_SMS_TO_OTP;

class WSOL_SMS_TO_Admin {

	public $wpsmstootp;
	protected $db;
	protected $tb_prefix;
	protected $options;

	public function __construct() {
		global $wpdb;

		$this->db        = $wpdb;
		$this->tb_prefix = $wpdb->prefix;
		$this->options   = WSOL_SMS_TO_Option::getOptions();
		$this->init();

		// Add Actions
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_assets' ) );
		add_action( 'admin_bar_menu', array( $this, 'admin_bar' ) );
                
                //check if any other SMSTO plugin has already added the action
                $hook_name = 'dashboard_glance_items';
                $search_value = 'dashboard_smsto_glance';
                global $wp_filter;

                if (isset($wp_filter[$hook_name])) {
                    $encoded_hook_name = json_encode($wp_filter[$hook_name]);
                    $no_of_occurances = substr_count($encoded_hook_name, $search_value);
                    if ($no_of_occurances == 0) {
                        add_action('dashboard_glance_items', array($this, 'dashboard_smsto_glance'));
                    }
                } else {
                    add_action('dashboard_glance_items', array($this, 'dashboard_smsto_glance'));
                }
            
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
                
		// Add Filters
		add_filter( 'plugin_row_meta', array( $this, 'meta_links' ), 0, 2 );
	}

	/**
	 * Include admin assets
	 */
	public function admin_assets() {
            
            
                wp_register_style( 'wpsmstootp-documentation_style', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/documentation_style.min.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
		wp_enqueue_style( 'wpsmstootp-documentation_style' );
                wp_enqueue_script( 'wpsmstootp-documentation_settings', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/documentation_settings.min.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
            
		// Register admin-bar.css for whole admin area
		wp_register_style( 'wpsmstootp-admin-bar', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/admin-bar.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
		wp_enqueue_style( 'wpsmstootp-admin-bar' );

		if ( stristr( get_current_screen()->id, "wp-sms-to-otp" ) ) {
			wp_register_style( 'wpsmstootp-admin', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/admin.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_style( 'wpsmstootp-admin' );
			if ( is_rtl() ) {
				wp_enqueue_style( 'wpsmstootp-rtl', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/rtl.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
			}

			wp_enqueue_style( 'wpsmstootp-chosen', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/chosen.min.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_script( 'wpsmstootp-chosen', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/chosen.jquery.min.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_script( 'wpsmstootp-word-and-character-counter', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/jquery.word-and-character-counter.min.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_script( 'wpsmstootprepeater', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/jquery.repeater.min.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
                        wp_enqueue_script( 'wpsmstootpblocktimerepeater', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/jquery.repeater.min.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_script( 'wpsmstootp-admin', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/admin.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
		}
	}

	/**
	 * Admin bar plugin
	 */
	public function admin_bar() {
		global $wp_admin_bar;
		if ( is_super_admin() && is_admin_bar_showing() ) {                   
                    $credit = get_option('wpsmstootp_gateway_credit');
                    if ($credit AND isset($this->options['account_credit_in_menu']) AND!is_object($credit)) {
                        $wp_admin_bar->add_node(
                                    array(
                                        'id' => 'wp-credit-sms',
                                        'parent' => 'top-secondary',
                                        'title' => '<span class="ab-icon"></span>' . '<span id="wp-credit-sms-title">'.$credit.'</span>',
                                        'href' => WSOL_WP_SMS_TO_OTP_ADMIN_URL . '/admin.php?page=wp-sms-to-otp-settings&tab=gateway'
                                    )
                        );
                    }
            }

		$wp_admin_bar->add_menu( array(
			'id'     => 'wp-send-sms',
			'parent' => 'new-content',
			'title'  => __( 'SMSTO', 'wp-sms-to-otp' ),
			'href'   => WSOL_WP_SMS_TO_OTP_ADMIN_URL . '/admin.php?page=wp-sms-to-otp'
		) );
	}
        
	/**
	 * Dashboard glance plugin
	 */
	public function dashboard_smsto_glance() {
		$subscribe = $this->db->get_var( "SELECT COUNT(*) FROM `{$this->db->prefix}usermeta` where `meta_key` = 'wp_capabilities' and `meta_value` LIKE '%ubscriber%'" );
		$credit    = get_option( 'wpsmstootp_gateway_credit' );

		echo "<li class='wpsmstootp-subscribe-count'><a href='" . WSOL_WP_SMS_TO_OTP_ADMIN_URL . "users.php?role=subscriber'>" . sprintf( __( '%s WP Subscriber(s)', 'wp-sms-to-otp' ), esc_html($subscribe) ) . "</a></li>";
		if ( ! is_object( $credit ) ) {
			echo "<li class='wpsmstootp-credit-count'><a href='" . WSOL_WP_SMS_TO_OTP_ADMIN_URL . "admin.php?page=wp-sms-to-otp-settings&tab=gateway'>" . sprintf( __( '%s SMSto Credit', 'wp-sms-to-otp' ), esc_html($credit) ) . "</a></li>";

		}
	}

	/**
	 * Administrator admin_menu
	 */
	public function admin_menu() {
		$hook_suffix = array();
		add_menu_page( __( 'WP - SMS OTP Login - by SMS.to', 'wp-sms-to-otp' ), __( 'SMS OTP Login', 'wp-sms-to-otp' ), 'wpsmstootp_login_register_action', 'wp-sms-to-otp', array( $this, 'login_register_action_callback' ), 'dashicons-smartphone' );
		add_submenu_page( 'wp-sms-to-otp', __( 'OTP Actions', 'wp-sms-to-otp' ), __( 'OTP Actions', 'wp-sms-to-otp' ), 'wpsmstootp_outbox', 'wp-sms-to-otp-login_register_action', array( $this, 'login_register_action_callback' ) );
                add_submenu_page( 'wp-sms-to-otp', __( 'Reports', 'wp-sms-to-otp' ), __( 'Reports', 'wp-sms-to-otp' ), 'wpsmstootp_outbox', 'wp-sms-to-otp-outbox', array( $this, 'outbox_callback' ) );
                add_submenu_page( 'wp-sms-to-otp', __( 'Documentation', 'wp-sms-to-otp' ), __( 'Documentation', 'wp-sms-to-otp' ), 'wpsmstootp_outbox', 'wp-sms-to-otp-documentation', array( $this, 'documentation_callback' ) );
                
		// Check GDPR compliance for Privacy menu
		if ( isset( $this->options['gdpr_wpsmstootp_compliance'] ) and $this->options['gdpr_wpsmstootp_compliance'] == 1 ) {
			$hook_suffix['privacy'] = add_submenu_page( 'wp-sms-to-otp', __( 'Privacy', 'wp-sms-to-otp' ), __( 'Privacy', 'wp-sms-to-otp' ), 'manage_options', 'wp-sms-to-otp-subscribers-privacy', array( $this, 'privacy_callback' ) );
		}

                // Add styles to menu pages
                foreach ($hook_suffix as $menu => $hook) {
                    add_action("load-{$hook}", array($this, $menu . '_assets'));
                }
        }



    /**
	 * Callback outbox page.
	 */
	public function outbox_callback() {
		$page = new Outbox();
		$page->render_page();
	}
        
    /**
	 * Callback outbox page.
	 */
	public function login_register_action_callback() {
		$page = new Login_register_action();
		$page->render_page();
	}      
        
    /**
	 * Callback outbox page.
	 */
	public function documentation_callback() {
            require_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/admin/documentation/class-wpsmstootp-documentation.php';
            
		$page = new WSOL_SMS_TO_Documentation();
		$page->render_page();
	}            

	/**
	 * Callback subscribers page.
	 */
	public function privacy_callback() {
		// Privacy class.
		require_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/admin/privacy/class-wpsmstootp-privacy.php';

		$page           = new WSOL_SMS_TO_Privacy();
		$page->pagehook = get_current_screen()->id;
		$page->render_page();
	}

	/**
	 * Load send SMS page assets
	 */
	public function send_sms_assets() {
			wp_enqueue_style( 'jquery-flatpickr', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/flatpickr.min.css', true, WSOL_WP_SMS_TO_OTP_VERSION );
			wp_enqueue_script( 'jquery-flatpickr', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/flatpickr.min.js', array( 'jquery' ), WSOL_WP_SMS_TO_OTP_VERSION );
	}

	/**
	 * Load privacy page assets
	 */
	public function privacy_assets() {
		$pagehook = get_current_screen()->id;

		wp_enqueue_script( 'common' );
		wp_enqueue_script( 'wp-lists' );
		wp_enqueue_script( 'postbox' );
                
                wp_register_script( 'wpsmstootp-privacy', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/privacy.js', true, WSOL_WP_SMS_TO_OTP_VERSION );
                wp_enqueue_script( 'wpsmstootp-privacy');                

		add_meta_box( 'privacy-meta-1', esc_html( get_admin_page_title() ), array( WSOL_SMS_TO_Privacy::class, 'privacy_meta_html_gdpr' ), $pagehook, 'side', 'core' );
		add_meta_box( 'privacy-meta-2', __( 'Export User’s Data related to WP - SMS OTP Login - by SMS.to', 'wp-sms-to-otp' ), array( WSOL_SMS_TO_Privacy::class, 'privacy_meta_html_export' ), $pagehook, 'normal', 'core' );
		add_meta_box( 'privacy-meta-3', __( 'Erase User’s Data related to WP - SMS OTP Login - by SMS.to', 'wp-sms-to-otp' ), array( WSOL_SMS_TO_Privacy::class, 'privacy_meta_html_delete' ), $pagehook, 'normal', 'core' );
	}

	/**
	 * Administrator add Meta Links
	 *
	 * @param $links
	 * @param $file
	 *
	 * @return array
	 */
	public function meta_links( $links, $file ) {
		if ( $file == 'wp-sms-to-otp/wp-sms-to-otp.php' ) {
			$rate_url = 'http://wordpress.org/support/view/plugin-reviews/wp-sms-to-otp?rate=5#postform';
			$links[]  = '<a href="' . $rate_url . '" target="_blank" class="wpsmstootp-plugin-meta-link" title="' . __( 'Click here to rate and review this plugin on WordPress.org', 'wp-sms-to-otp' ) . '">' . __( 'Rate this plugin', 'wp-sms-to-otp' ) . '</a>';

		//	$newsletter_wpsmstootp_url = WSOL_WP_SMS_TO_OTP_SITE . '/newsletter';
		//	$links[]        = '<a href="' . $newsletter_wpsmstootp_url . '" target="_blank" class="wpsmstootp-plugin-meta-link" title="' . __( 'Click here to rate and review this plugin on WordPress.org', 'wp-sms-to-otp' ) . '">' . __( 'Subscribe to our Email Newsletter', 'wp-sms-to-otp' ) . '</a>';
		}

		return $links;
	}

	/**
	 * Adding new capability in the plugin
	 */
	public function add_cap() {
		// Get administrator role
		$role = get_role( 'administrator' );

		$role->add_cap( 'wpsmstootp_sendsms' );
		$role->add_cap( 'wpsmstootp_outbox' );
		$role->add_cap( 'wpsmstootp_subscribers' );
		$role->add_cap( 'wpsmstootp_setting' );
	}

	/**
	 * Initial plugin
	 */
	private function init() {
		if ( isset( $_GET['action'] ) ) {
			if ( sanitize_text_field($_GET['action'] == 'wpsmstootp-hide-newsletter' )) {
				update_option( 'wpsmstootp_hide_newsletter', true );
			}
		}

		if ( ! get_option( 'wpsmstootp_hide_newsletter' ) ) {
			//add_action( 'sms_to_otp_settings_page', array( $this, 'admin_newsletter' ) );
		}

		// Check exists require function
		if ( ! function_exists( 'wp_get_current_user' ) ) {
			include( ABSPATH . "wp-includes/pluggable.php" );
		}

		// Add plugin caps to admin role
		if ( is_admin() and is_super_admin() ) {
			$this->add_cap();
		}
	}

}

new WSOL_SMS_TO_Admin();