<?php

namespace WSOL_WP_SMS_TO_OTP;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // No direct access allowed ;)

class WSOL_SMS_TO_Settings {

	public $setting_name;
	public $options = array();

	public function __construct() {
		$this->setting_name = 'wpsmstootp_settings';
		$this->get_settings();
		$this->options = get_option( $this->setting_name );
              
		if ( empty( $this->options ) ) {
			update_option( $this->setting_name, array() );
		}
                
		add_action( 'admin_menu', array( $this, 'add_settings_menu' ), 11 );

		if ( isset( $_GET['page'] ) and sanitize_text_field($_GET['page']) == 'wp-sms-to-otp-settings' or isset( $_POST['option_page'] ) and sanitize_text_field($_POST['option_page']) == 'wpsmstootp_settings' ) {
			add_action( 'admin_init', array( $this, 'register_settings' ) );
		}
                              
	}

	/**
	 * Add WP-SMSto Package admin page settings
	 * */
	public function add_settings_menu() {
		add_submenu_page( 'wp-sms-to-otp', __( 'Settings', 'wp-sms-to-otp' ), __( 'Settings', 'wp-sms-to-otp' ), 'wpsmstootp_setting', 'wp-sms-to-otp-settings', array(
			$this,
			'render_settings'
		) );
	}

	/**
	 * Gets saved settings from WP core
	 *
	 * @since           2.0
	 * @return          array
	 */
	public function get_settings() {
		$settings = get_option( $this->setting_name );
                
               $url = site_url();
         
		if ( ! $settings ) {
			update_option( $this->setting_name, array(
                            'gateway_wpsmstootp_sender_id' => 'SMSto',
                            'login_sms' => 1,
                            'international_mobile_phone' => 1,
                            'gateway_wpsmstootp_callback_url' => $url, 
                            'password_validity_value' => 60,
                            'login_no_of_retries_value' => 3,
                            'password_validation_no_of_retries_value' => 3,
                            'add_mobile_phone_field' => 1,
                            'account_credit_in_menu' => 1,
			) );        
		}
		return apply_filters( 'wpsmstootp_get_settings', $settings );
	}

	/**
	 * Registers settings in WP core
	 *
	 * @since           2.0
	 * @return          void
	 */
	public function register_settings() {
		if ( false == get_option( $this->setting_name ) ) {
			add_option( $this->setting_name );
		}

		foreach ( $this->get_registered_settings() as $tab => $settings ) {
			add_settings_section(
				'wpsmstootp_settings_' . $tab,
				__return_null(),
				'__return_false',
				'wpsmstootp_settings_' . $tab
			);                        
               
			if ( empty( $settings ) ) {
				return;
			}

			foreach ( $settings as $option ) {
				$name = isset( $option['name'] ) ? $option['name'] : '';

				add_settings_field(
					'wpsmstootp_settings[' . $option['id'] . ']',
					$name,
					array( $this, $option['type'] . '_callback' ),
					'wpsmstootp_settings_' . $tab,
					'wpsmstootp_settings_' . $tab,
					array(
						'id'      => isset( $option['id'] ) ? $option['id'] : null,
						'desc'    => ! empty( $option['desc'] ) ? $option['desc'] : '',
						'name'    => isset( $option['name'] ) ? $option['name'] : null,
						'section' => $tab,
						'size'    => isset( $option['size'] ) ? $option['size'] : null,
						'options' => isset( $option['options'] ) ? $option['options'] : '',
						'std'     => isset( $option['std'] ) ? $option['std'] : ''
					)
				);
                        
				register_setting( $this->setting_name, $this->setting_name, array( $this, 'settings_sanitize' ) );
			}
		}
                 }
                 
	/**
	 * Gets settings tabs
	 *
	 * @since               2.0
	 * @return              array Tabs list
	 */
	public function get_tabs() {
		$tabs = array(
			'general'       => __( 'General', 'wp-sms-to-otp' ),
			'gateway'       => __( 'Gateway', 'wp-sms-to-otp' ),
			'feature'       => __( 'Features', 'wp-sms-to-otp' ),
		);

		return $tabs;
	}

	/**
	 * Sanitizes and saves settings after submit
	 *
	 * @since               2.0
	 *
	 * @param               array $input Settings input
	 *
	 * @return              array New settings
	 */
	public function settings_sanitize( $input = array() ) {            

		if ( empty( $_POST['_wp_http_referer'] ) ) {
			return $input;
		}

		parse_str( sanitize_text_field($_POST['_wp_http_referer']), $referrer );

		$settings = $this->get_registered_settings();
		$tab      = isset( $referrer['tab'] ) ? $referrer['tab'] : 'wp';

		$input = $input ? $input : array();
		$input = apply_filters( 'wpsmstootp_settings_' . $tab . '_sanitize', $input );

		// Loop through each setting being saved and pass it through a sanitization filter
		foreach ( $input as $key => $value ) {

			// Get the setting type (checkbox, select, etc)
			$type = isset( $settings[ $tab ][ $key ]['type'] ) ? $settings[ $tab ][ $key ]['type'] : false;

			if ( $type ) {
				// Field type specific filter
				$input[ $key ] = apply_filters( 'wpsmstootp_settings_sanitize_' . $type, $value, $key );
			}

			// General filter
			$input[ $key ] = apply_filters( 'wpsmstootp_settings_sanitize', $value, $key );
		}

		// Loop through the whitelist and unset any that are empty for the tab being saved
		if ( ! empty( $settings[ $tab ] ) ) {
			foreach ( $settings[ $tab ] as $key => $value ) {

				// settings used to have numeric keys, now they have keys that match the option ID. This ensures both methods work
				if ( is_numeric( $key ) ) {
					$key = $value['id'];
				}

				if ( empty( $input[ $key ] ) ) {
					unset( $this->options[ $key ] );
				}

			}
		}

		// Merge our new settings with the existing
		$output = array_merge( $this->options, $input );

		add_settings_error( 'wpsmstootp-notices', '', __( 'Settings updated', 'wp-sms-to-otp' ), 'updated' );

		return $output;

	}

	/**
	 * Get settings fields
	 *
	 * @since           2.0
	 * @return          array Fields
	 */
	public function get_registered_settings() {

		$options = array(
			'enable'  => __( 'Enable', 'wp-sms-to-otp' ),
			'disable' => __( 'Disable', 'wp-sms-to-otp' )
		);

               
		$settings = apply_filters( 'sms_to_otp_registered_settings', array(
			// General tab
			'general'       => apply_filters( 'sms_to_otp_general_settings', array(
				'admin_title_wpsmstootp_privacy' => array(
					'id'   => 'admin_title_wpsmstootp_privacy',
					'name' => __( 'Privacy', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'gdpr_wpsmstootp_compliance'     => array(
					'id'      => 'gdpr_wpsmstootp_compliance',
					'name'    => __( 'GDPR Enhancements', 'wp-sms-to-otp' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'Enable GDPR related features in this page. Read our GDPR documentation to learn more.', 'wp-sms-to-otp' ),
				),
			) ),

			// Gateway tab
			'gateway'       => apply_filters( 'sms_to_otp_gateway_settings', array(
				// Gateway
				'gayeway_title'             => array(
					'id'   => 'gayeway_title',
					'name' => __( 'Gateway information', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'gateway_wpsmstootp_api_key'          => array(
					'id'   => 'gateway_wpsmstootp_api_key',
					'name' => __( 'API Key', 'wp-sms-to-otp' ),
					'type' => 'text',
					'desc' => __( 'Enter Api Key', 'wp-sms-to-otp' )
				),  
                                'gateway_wpsmstootp_callback_url'          => array(
					'id'   => 'gateway_wpsmstootp_callback_url',
					'name' => __( 'Callback URL', 'wp-sms-to-otp' ),
					'type' => 'text',
					'desc' => __( 'Your WordPress callback URL to update Reports<br>If you must change it, please do it with caution', 'wp-sms-to-otp' )
				),                                                        
				'gateway_wpsmstootp_sender_id'         => array(
					'id'   => 'gateway_wpsmstootp_sender_id',
					'name' => __( 'Sender', 'wp-sms-to-otp' ),
					'type' => 'text11chars',
					'std'  => WSOL_SMS_TO_Gateway::from(),
					'desc' => __( 'Sender number or sender ID - 11 characters max.', 'wp-sms-to-otp' )
				),
				'wpsmstootp_gateway_key'               => array(
					'id'   => 'wpsmstootp_gateway_key',
					'name' => __( 'API key', 'wp-sms-to-otp' ),
					'type' => 'text',
					'desc' => __( 'Enter API key of gateway', 'wp-sms-to-otp' )
				),
				// Gateway status
				'wpsmstootp_gateway_status_title'      => array(
					'id'   => 'wpsmstootp_gateway_status_title',
					'name' => __( 'Gateway status', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'account_credit'            => array(
					'id'      => 'account_credit',
					'name'    => __( 'Status', 'wp-sms-to-otp' ),
					'type'    => 'html',
					'options' => WSOL_SMS_TO_Gateway::status(),
				),
				'account_response'          => array(
					'id'      => 'account_response',
					'name'    => __( 'Result request', 'wp-sms-to-otp' ),
					'type'    => 'html',
					'options' => WSOL_SMS_TO_Gateway::response(),
				),
				'bulk_send'                 => array(
					'id'      => 'bulk_send',
					'name'    => __( 'Bulk send', 'wp-sms-to-otp' ),
					'type'    => 'html',
					'options' => WSOL_SMS_TO_Gateway::bulk_status(),
				),
				// Account credit
				'account_credit_title'      => array(
					'id'   => 'account_credit_title',
					'name' => __( 'Account balance', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'account_credit_in_menu'    => array(
					'id'      => 'account_credit_in_menu',
					'name'    => __( 'Show in admin menu', 'wp-sms-to-otp' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'Show your account credit in admin menu.', 'wp-sms-to-otp' )
				),
			) ),

			
			// Feature tab
			'feature'       => apply_filters( 'sms_to_otp_feature_settings', array(
				'mobile_phone_field'                     => array(
					'id'   => 'mobile_phone_field',
					'name' => __( 'Mobile field', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'add_mobile_phone_field'                 => array(
					'id'      => 'add_mobile_phone_field',
					'name'    => __( 'Add Mobile number field', 'wp-sms-to-otp' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'Add Mobile number to user profile and register form.', 'wp-sms-to-otp' )
				),
                                'login_title'       => array(
					'id'   => 'login_title',
					'name' => __( 'Login', 'wp-sms' ),
					'type' => 'header'
				),
				'login_sms'         => array(
					'id'      => 'login_sms',
					'name'    => __( 'Login with mobile', 'wp-sms' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'This option adds login with SMS in the login form.', 'wp-sms' ),
				),
				'international_mobile_phone_title'       => array(
					'id'   => 'international_mobile_phone_title',
					'name' => __( 'International Telephone Input', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'international_mobile_phone'             => array(
					'id'      => 'international_mobile_phone',
					'name'    => __( 'Enable for mobile phone fields', 'wp-sms-to-otp' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'Adds country code in mobile phone field', 'wp-sms-to-otp' )
				),
				'international_mobile_phone_only_countries'      => array(
					'id'      => 'international_mobile_phone_only_countries',
					'name'    => __( 'Only Countries', 'wp-sms-to-otp' ),
					'type'    => 'countryselect',
					'options' => $this->get_countries_list(),
					'desc'    => __( 'In the dropdown Country select display only the countries you specify.', 'wp-sms-to-otp' )
				),
				'international_mobile_phone_preferred_countries' => array(
					'id'      => 'international_mobile_phone_preferred_countries',
					'name'    => __( 'Prefix Countries', 'wp-sms-to-otp' ),
					'type'    => 'countryselect',
					'options' => $this->get_countries_list(),
					'desc'    => __( 'Specify the countries to appear at the top of the list.', 'wp-sms-to-otp' )
				),
				'password_validity'             => array(
					'id'   => 'password_validity',
					'name' => __( 'Password Validity', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'password_validity_value'          => array(
					'id'   => 'password_validity_value',
					'name' => __( 'Password validity value', 'wp-sms-to-otp' ),
					'type' => 'number',
					'desc' => __( 'Enter Password Validity in seconds', 'wp-sms-to-otp' )
				),         
				'login_no_of_retries'             => array(
					'id'   => 'password_no_of_retries',
					'name' => __( 'Login No of retries', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'login_no_of_retries_value'          => array(
					'id'   => 'login_no_of_retries_value',
					'name' => __( 'Login No of retries value', 'wp-sms-to-otp' ),
					'type' => 'number',
					'desc' => __( 'Enter the maximum number of login tries', 'wp-sms-to-otp' )
				),                              
				'password_validation_no_of_retries'             => array(
					'id'   => 'password_validation_no_of_retries',
					'name' => __( 'Password Validation', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'password_validation_no_of_retries_value'          => array(
					'id'   => 'password_validation_no_of_retries_value',
					'name' => __( 'Password Validation No of retries value', 'wp-sms-to-otp' ),
					'type' => 'number',
					'desc' => __( 'Enter the maximum number of allow failed passwords', 'wp-sms-to-otp' )
				),                                      
                                'block_time_id'           => array(
					'id'   => 'block_time_id',
					'name' => __( 'User Block Time', 'wp-sms-to-otp' ),
					'type' => 'header'
				),
				'block_time_enable'    => array(
					'id'      => 'block_time_enable',
					'name'    => __( 'Status', 'wp-sms-to-otp' ),
					'type'    => 'checkbox',
					'options' => $options,
					'desc'    => __( 'User Block Time Enable', 'wp-sms-to-otp' )
				),
				'block_time_content'    => array(
					'id'      => 'block_time_content',
					'name'    => __( 'Block Time', 'wp-sms-to-otp' ),
					'type'    => 'wpsmstootpblocktimerepeater',
					'desc'    => __( 'Add Block time for each user group upon login/password authentication failure', 'wp-sms-to-otp' )
				),                                                         
			) ),
			
		) );

		return $settings;
	}

	public function header_callback( $args ) {
		echo '<hr/>';
	}

	public function html_callback( $args ) {
		echo $args['options'];
	}

	public function notice_callback( $args ) {
		echo $args['desc'];
	}

	public function checkbox_callback( $args ) {
		$checked = isset( $this->options[ $args['id'] ] ) ? checked( 1, $this->options[ $args['id'] ], false ) : '';
		$html    = '<input type="checkbox" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="1" ' . $checked . '/>';
		$html    .= '<label for="wpsmstootp_settings[' . $args['id'] . ']"> ' . __( 'Active', 'wp-sms-to-otp' ) . '</label>';
		$html    .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function multicheck_callback( $args ) {
		$html = '';
		foreach ( $args['options'] as $key => $value ) {
			$option_name = $args['id'] . '-' . $key;
			$this->checkbox_callback( array(
				'id'   => $option_name,
				'desc' => $value
			) );
			echo '<br>';
		}

		echo $html;
	}

	public function radio_callback( $args ) {
		foreach ( $args['options'] as $key => $option ) :
			$checked = false;

			if ( isset( $this->options[ $args['id'] ] ) && $this->options[ $args['id'] ] == $key ) {
				$checked = true;
			} elseif ( isset( $args['std'] ) && $args['std'] == $key && ! isset( $this->options[ $args['id'] ] ) ) {
				$checked = true;
			}

			echo '<input name="wpsmstootp_settings[' . $args['id'] . ']"" id="wpsmstootp_settings[' . $args['id'] . '][' . $key . ']" type="radio" value="' . $key . '" ' . checked( true, $checked, false ) . '/>';
			echo '<label for="wpsmstootp_settings[' . $args['id'] . '][' . $key . ']">' . $option . '</label>&nbsp;&nbsp;';
		endforeach;

		echo '<p class="description">' . $args['desc'] . '</p>';
	}

	public function text11chars_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) and $this->options[ $args['id'] ] ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

                $maxlength="11";
       
		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input required type="text" class="' . $size . '-text" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( stripslashes( $value ) ) . '" maxlength="' . esc_attr( $maxlength ) . '"/>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}        
	public function text_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) and $this->options[ $args['id'] ] ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="text" class="' . $size . '-text" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( stripslashes( $value ) ) . '"/>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}
        
        
        
	public function international_phone_number_callback( $args ) {
   
            echo '<hr/>';
            
		if ( isset( $this->options[ $args['id'] ] ) and $this->options[ $args['id'] ] ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}
                
                

                if ( sms_to_otp_get_option( 'international_mobile_phone' ) ) {
                        $sms_to_otp_input_mobile_phone = " wp-sms-to-otp-input-mobile_phone";
                } else {
                        $sms_to_otp_input_mobile_phone = "";
                }                 

		$html = '<input type="text" class="regular-text' . $sms_to_otp_input_mobile_phone . '" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( stripslashes( $value ) ) . '"/>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;


                        

	}        

	public function number_callback( $args ) {
            
                $style="width:15%";
                $required="true";
            
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$max  = isset( $args['max'] ) ? $args['max'] : 999999;
		$min  = isset( $args['min'] ) ? $args['min'] : 1;
		$step = isset( $args['step'] ) ? $args['step'] : 1;

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="number" step="' . esc_attr( $step ) . '" required="' . esc_attr( $required ) . '" style="' . esc_attr( $style ). '" max="' . esc_attr( $max ) . '" min="' . esc_attr( $min ) . '" class="' . $size . '-text" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( stripslashes( $value ) ) . '"/>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function textarea_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<textarea class="large-text" cols="50" rows="5" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']">' . esc_textarea( stripslashes( $value ) ) . '</textarea>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function password_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="password" class="' . $size . '-text" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( $value ) . '"/>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function missing_callback( $args ) {
		echo '&ndash;';

		return false;
	}


	public function select_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$html = '<select id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']"/>';

		foreach ( $args['options'] as $option => $name ) :
			$selected = selected( $option, $value, false );
			$html     .= '<option value="' . $option . '" ' . $selected . '>' . $name . '</option>';
		endforeach;

		$html .= '</select>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function multiselect_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$html     = '<select id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . '][]" multiple="true" class="chosen-select"/>';
		$selected = '';

		foreach ( $args['options'] as $k => $name ) :
			foreach ( $name as $option => $name ):
				if ( isset( $value ) AND is_array( $value ) ) {
					if ( in_array( $option, $value ) ) {
						$selected = " selected='selected'";
					} else {
						$selected = '';
					}
				}
				$html .= '<option value="' . $option . '" ' . $selected . '>' . $name . '</option>';
			endforeach;
		endforeach;

		$html .= '</select>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function countryselect_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$html     = '<select id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . '][]" multiple="true" class="chosen-select"/>';
		$selected = '';

		foreach ( $args['options'] as $option => $country ) :
			if ( isset( $value ) AND is_array( $value ) ) {
				if ( in_array( $country['code'], $value ) ) {
					$selected = " selected='selected'";
				} else {
					$selected = '';
				}
			}
			$html .= '<option value="' . $country['code'] . '" ' . $selected . '>' . $country['name'] . '</option>';
		endforeach;

		$html .= '</select>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}


	public function advancedselect_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		if ( is_rtl() ) {
			$class_name = 'chosen-select chosen-rtl';
		} else {
			$class_name = 'chosen-select';
		}

		$html = '<select class="' . $class_name . '" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']"/>';

		foreach ( $args['options'] as $key => $v ) {
			$html .= '<optgroup label="' . ucfirst( str_replace( '_', ' ', $key ) ) . '">';

			foreach ( $v as $option => $name ) :
				$disabled = ( $key == 'pro_pack_gateways' ) ? $disabled = ' disabled' : '';
				$selected = selected( $option, $value, false );
				$html     .= '<option value="' . $option . '" ' . $selected . ' ' . $disabled . '>' . ucfirst( $name ) . '</option>';
			endforeach;

			$html .= '</optgroup>';
		}

		$html .= '</select>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function color_select_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$html = '<select id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']"/>';

		foreach ( $args['options'] as $option => $color ) :
			$selected = selected( $option, $value, false );
			$html     .= '<option value="' . $option . '" ' . $selected . '>' . $color['label'] . '</option>';
		endforeach;

		$html .= '</select>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function rich_editor_callback( $args ) {
		global $wp_version;

		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		if ( $wp_version >= 3.3 && function_exists( 'wp_editor' ) ) {
			$html = wp_editor( stripslashes( $value ), 'wpsmstootp_settings[' . $args['id'] . ']', array( 'textarea_name' => 'wpsmstootp_settings[' . $args['id'] . ']' ) );
		} else {
			$html = '<textarea class="large-text" rows="10" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']">' . esc_textarea( stripslashes( $value ) ) . '</textarea>';
		}

		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function upload_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="text" class="' . $size . '-text wpsmstootp_upload_field" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( stripslashes( $value ) ) . '"/>';
		$html .= '<span>&nbsp;<input type="button" class="wpsmstootp_settings_upload_button button-secondary" value="' . __( 'Upload File', 'wpsmstootp' ) . '"/></span>';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}

	public function color_callback( $args ) {
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		$default = isset( $args['std'] ) ? $args['std'] : '';

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="text" class="wpsmstootp-color-picker" id="wpsmstootp_settings[' . $args['id'] . ']" name="wpsmstootp_settings[' . $args['id'] . ']" value="' . esc_attr( $value ) . '" data-default-color="' . esc_attr( $default ) . '" />';
		$html .= '<p class="description"> ' . $args['desc'] . '</p>';

		echo $html;
	}
        
        
	public function wpsmstootpblocktimerepeater_callback($args)
	{
		if ( isset( $this->options[ $args['id'] ] ) ) {
			$value = $this->options[ $args['id'] ];
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}
                
                
                
                $user_roles = wp_roles()->roles;
		ob_start();
		?>
		<div class="wpsmstootpblocktimerepeater" id="wpsmstootpblocktimerepeater">
			<div data-repeater-list="wpsmstootp_settings[<?php echo $args['id'] ?>]">
				<?php if(is_array($value) && count($value)){ ?>
					<?php foreach($value as $data){ ?>
                                                <?php $user_role = isset($data['user_role']) ? $data['user_role'] : '' ?>
                                                <?php $block_time = isset($data['block_time']) ? $data['block_time'] : 1 ?>
						<div class="repeater-item" data-repeater-item>
							<div style="display: block; width: 100%; margin-bottom: 15px; border-bottom: 1px solid #ccc;">
								<div style="display: block; width: 30%; float: left; margin-bottom: 15px;">
									<select name="user_role" style="display: block; width: 100%;">
										<option value="">- Please Choose -</option>
										<?php foreach ($user_roles as $user_role_key => $user_role_name) { ?>
										<option value="<?= $user_role_key ?>" <?= ($user_role == $user_role_key) ? 'selected' : '' ?>><?= $user_role_name['name'] ?></option>
										<?php } ?>
									</select>
									<p class="description">Please set block time for each role</p>
								</div>           
                                                                        <div style="display: block; width: 70%; float: left; margin-bottom: 15px;">
                                                                                <input name="block_time" min="1" max="999999" step="1" required="true" value= "<?php echo esc_html($block_time) ?>"  type="number" style="display: block; width: 20%;">
                                                                                <p class="description">Please enter the block time in minutes</p>
                                                                        </div>                                                              
        							<div>
									<input type="button" value="Delete" class="button" style="margin-bottom: 15px;" data-repeater-delete />
								</div>
							</div>
						</div>
					<?php } ?>
				<?php } else { ?>
					<div class="repeater-item" data-repeater-item>
						<div style="display: block; width: 100%; margin-bottom: 15px; border-bottom: 1px solid #ccc;">
							<div style="display: block; width: 30%; float: left; margin-bottom: 15px;">
								<select name="user_role" style="display: block; width: 100%;">
									<option value="">- Please Choose -</option>
									<?php  foreach ($user_roles as $user_role_key => $user_role_name) { ?>
										<option value="<?= $user_role_key ?>"><?= $user_role_name ['name'] ?></option>
									<?php } ?>
								</select>
								<p class="description">Please choose a Role</p>
							</div>   
							<div style="display: block; width: 70%; float: left; margin-bottom: 15px;">
                                                            <input name="block_time" min="1" max="999999" step="1" required="true" value= "<?php echo isset($block_time) ? esc_html($block_time) : 1 ?>" type="number" style="display: block; width: 20%;">
								<p class="description">Please enter the block time in minutes</p>
							</div>                                                    
							<div>
								<input type="button" value="Delete" class="button" style="margin-bottom: 15px;" data-repeater-delete />
							</div>
						</div>
					</div>
				<?php } ?>
			</div>
			<div style="margin: 10px 0;">
				<input type="button" value="Add another block time" class="button button-primary" data-repeater-create />
			</p>
		</div>
		<?php
		echo ob_get_clean();
	}           
        
	public function render_settings() {           
		$active_tab = isset( $_GET['tab'] ) && array_key_exists( $_GET['tab'], $this->get_tabs() ) ? sanitize_text_field($_GET['tab']) : 'general';

		ob_start();
		?>
        <div class="wrap wpsmstootp-settings-wrap">
			<?php do_action( 'sms_to_otp_settings_page' ); ?>
            <h2><?php _e( 'Settings', 'wp-sms-to-otp' ) ?></h2>
            <div class="wpsmstootp-tab-group">
                <ul class="wpsmstootp-tab">
                    <li id="wpsmstootp-logo" class="wpsmstootp-logo-group">
                        <img src="<?php echo WSOL_WP_SMS_TO_OTP_URL; ?>assets/images/intergo-telecom-logo.png"/>
                        <p><?php echo sprintf( __( 'WP - SMS OTP Login - by SMS.to v%s', 'wp-sms-to-otp' ), WSOL_WP_SMS_TO_OTP_VERSION ); ?></p>
						<?php do_action( 'sms_to_otp_after_setting_logo' ); ?>
                    </li>              
					<?php                                    
					foreach ( $this->get_tabs() as $tab_id => $tab_name ) {

						$tab_url = add_query_arg( array(
							'settings-updated' => false,
							'tab'              => $tab_id
						) );

						$active = $active_tab == $tab_id ? 'active' : '';
                                              
						echo '<li><a href="' . esc_url( $tab_url ) . '" title="' . esc_attr( $tab_name ) . '" class="' . $active . '">';
						echo $tab_name;
						echo '</a></li>';
					}
					?>
                </ul>
				<?php echo settings_errors( 'wpsmstootp-notices' ); ?>
                <div class="wpsmstootp-tab-content">
                    <form method="post" action="options.php">
                        <table class="form-table">
							<?php
							settings_fields( $this->setting_name );
							do_settings_fields( 'wpsmstootp_settings_' . $active_tab, 'wpsmstootp_settings_' . $active_tab );
							?>
                        </table>
						<?php submit_button(); ?>
                    </form>
                </div>
            </div>
        </div>
		<?php
		echo ob_get_clean();
	}

	/*
	 * Get list Post Type
	 */
	public function get_list_post_type( $args = array() ) {

		// vars
		$post_types = array();

		// extract special arg
		$exclude   = array();
		$exclude[] = 'attachment';
		$exclude[] = 'acf-field'; //Advance custom field
		$exclude[] = 'acf-field-group'; //Advance custom field Group
		$exclude[] = 'vc4_templates'; //Visual composer
		$exclude[] = 'vc_grid_item'; //Visual composer Grid
		$exclude[] = 'acf'; //Advance custom field Basic
		$exclude[] = 'wpcf7_contact_form'; //contact 7 Post Type
		$exclude[] = 'shop_order'; //WooCommerce Shop Order
		$exclude[] = 'shop_coupon'; //WooCommerce Shop coupon

		// get post type objects
		$objects = get_post_types( $args, 'objects' );
		foreach ( $objects as $k => $object ) {
			if ( in_array( $k, $exclude ) ) {
				continue;
			}
			if ( $object->_builtin && ! $object->public ) {
				continue;
			}
			$post_types[] = array( $object->cap->publish_posts . '|' . $object->name => $object->label );
		}

		// return
		return $post_types;
	}

	/**
	 * Get countries list
	 *
	 * @return array|mixed|object
	 */
	public function get_countries_list() {
		// Load countries list file
		$file   = WSOL_WP_SMS_TO_OTP_DIR . 'assets/countries.json';
		$file   = file_get_contents( $file );
		$result = json_decode( $file, true );

		return $result;
	}
        
       
        
}

new WSOL_SMS_TO_Settings();