<?php

namespace WSOL_WP_SMS_TO_OTP;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

class WSOL_SMS_TO_Features {

    public $wpsmstootp;
    public $date;
    public $options;
    protected $db;
    protected $tb_prefix;

    /**
     * WSOL_WP_SMS_TO_OTP_Features constructor.
     */
    public function __construct() {
        global $wpsmstootp, $wpdb;

        $this->sms = $wpsmstootp;
        $this->db = $wpdb;
        $this->tb_prefix = $wpdb->prefix;
        $this->date = WSOL_WP_SMS_TO_OTP_CURRENT_DATE;
        $this->options = WSOL_SMS_TO_Option::getOptions();
      

        if (isset($this->options['add_mobile_phone_field'])) {

            //check if any other SMSTO plugin has already added the action
            // check only for register form, if it applies then it applies also for 
            // user_new_form and user_contactmethods  
            
            $hook_name = 'register_form';
            $search_value = 'add_smsto_mobile_phone_field_to_register_form';
            global $wp_filter;
            if (isset($wp_filter[$hook_name])) {
                $encoded_hook_name = json_encode($wp_filter[$hook_name]);
                $no_of_occurances = substr_count($encoded_hook_name, $search_value);
                if ($no_of_occurances == 0) {
                    add_action('register_form', array($this, 'add_smsto_mobile_phone_field_to_register_form'));
                    add_action('user_new_form', array($this, 'add_smsto_mobile_phone_field_to_newuser_form'));
                    add_filter('user_contactmethods', array($this, 'add_smsto_mobile_phone_field_to_profile_form'));
                }
            } else {
                    add_action('register_form', array($this, 'add_smsto_mobile_phone_field_to_register_form'));
                    add_action('user_new_form', array($this, 'add_smsto_mobile_phone_field_to_newuser_form'));
                    add_filter('user_contactmethods', array($this, 'add_smsto_mobile_phone_field_to_profile_form'));
            }

            add_filter('registration_errors', array($this, 'registration_errors'), 10, 3);
            add_action('user_register', array($this, 'save_register'));

            add_action('user_register', array($this, 'check_admin_duplicate_number'));
            add_action('profile_update', array($this, 'check_admin_duplicate_number'));
        }

        if (isset($this->options['login_sms'])) {
            add_action('login_head', array($this, 'my_custom_login'));
            add_action('login_form', array($this, 'add_otp_in_login_form'), 10, 2);
            add_filter('authenticate', array($this, 'login_user'), 20, 3);
            add_action( 'login_enqueue_scripts', array($this, 'login_type'), 20, 1);

           
        }

        if (isset($this->options['international_mobile_phone'])) {
            add_action('wp_enqueue_scripts', array($this, 'load_international_input'));
            add_action('admin_enqueue_scripts', array($this, 'load_international_input'));
            add_action('login_enqueue_scripts', array($this, 'load_international_input'));
        }
        
        $page = isset($_GET['page']) ? $_GET['page'] : null;
        if (isset($page)) {
            if ($page == 'wp-sms-to-otp-outbox') {
                add_action('admin_enqueue_scripts', array($this, 'update_outbox_otp'));
                add_action('wp_enqueue_scripts', array($this, 'update_outbox_otp'), 10, 2);
            }
        }
    }

    
     /**
     * Add javascript for outbox
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     */
    
    function update_outbox_otp() {
        wp_register_script('update_outbox_otp', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/outbox-form-otp.js', true, WSOL_WP_SMS_TO_OTP_VERSION);
        wp_enqueue_script('update_outbox_otp', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/outbox-form-otp.js', true, WSOL_WP_SMS_TO_OTP_VERSION);
    }
    
    
    /**
     * Retrieve User using mobile phone
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     * @param mobile_phone, mobile_phone number
     * @return User
     */
    public function wsol_sms_to_get_user_by_mobile_phone_otp($db_field, $value) {
        global $wpdb;

        $user_id_obj = $wpdb->get_row(
                $wpdb->prepare(
                        "SELECT user_id FROM $wpdb->prefix" . "usermeta WHERE meta_key = %s
				 AND replace(meta_value, ' ', '') = +%d LIMIT 1",
                        $db_field, $value
                )
        );
        if (isset($user_id_obj)) {
            $user = get_user_by('id', $user_id_obj->user_id);
            return $user;
        } else {
            return null;
        }
    }

    /**
     * Login the user 
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     * @param mobile_phone, mobile_phone number
     * @return User
     */
    public function login_user($user, $username, $password) {

        global $wpdb;

        $ip = wsol_sms_to_getRealIpAddr();
       

       if (isset($_REQUEST['login_type']))   {
            $login_type = $wpdb->_escape($_REQUEST['login_type']);
            
            $_SESSION['login_type'] = $login_type;
            
            if ($login_type == 'username_password') {
                $username = $wpdb->_escape($_REQUEST['log']);
                $password = $wpdb->_escape($_REQUEST['pwd']);
                $user = get_user_by('login', $username);


                if (empty($username) && empty($password)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();
                    
                    if (empty($username) && empty($password)) {
                        $error->add('empty_password', __('<strong>Error</strong>: The Username/Email Address or Password fields are empty', 'actions-pack'));
                    }

                    return $error;
                }

                if (empty($username)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();

                    if (empty($username)) {
                        $error->add('empty_password', __('<strong>Error</strong>: The Username/Email Address field is empty', 'actions-pack'));
                    }

                    return $error;
                }
                if (empty($password)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();

                    if (empty($password)) {
                        $error->add('empty_password', __('<strong>Error</strong>: The Password field is empty', 'actions-pack'));
                    }

                    return $error;
                }

                /** This filter is documented in wp-includes/user.php */
                $user = apply_filters('wp_authenticate_user', $user, $password);

                if ($user == true) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    if (!wp_check_password($password, $user->user_pass, $user->ID)) {
                        return new \WP_Error(
                                'incorrect_password222',
                                sprintf(
                                        /* translators: %s: Phone Number. */
                                        __('<strong>Error</strong>: The password you entered %s is incorrect.', 'actions-pack'),
                                        '<strong>' . $username . '</strong>'
                                ) .
                                ' <a href="' . wp_lostpassword_url() . '">' .
                                __('Lost your password?') .
                                '</a>'
                        );
                    }
                    return $user;
                }
            }


            if ($login_type == 'otp') {

                if (isset($_REQUEST['mobile_phone'])) {
                    $mobile_phone_posted = $wpdb->_escape($_REQUEST['mobile_phone']);
                    $mobile_phone = str_replace(' ', '', $mobile_phone_posted);
                }

                if (isset($_REQUEST['otp_code'])) {
                    $otp_code = $wpdb->_escape($_REQUEST['otp_code']);
                } else {
                    $otp_code = '';
                }

                $user = $this->wsol_sms_to_get_user_by_mobile_phone_otp('mobile_phone', $mobile_phone);                

                if (($user == null) && empty($otp_code)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();

                    if (($user == null) && empty($otp_code)) {
                        $error->add('empty_password', __('<strong>Error</strong>: Password not authenticated', 'actions-pack'));
                    }

                    return $error;
                }

                if (($user == null)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();

                    if (($user == null)) {
                        $error->add('empty_password', __('<strong>Error</strong>: Password not authenticated', 'actions-pack'));
                    }

                    return $error;
                }
                if (empty($otp_code)) {
                    if (is_wp_error($user)) {
                        return $user;
                    }

                    $error = new \WP_Error();

                    if (empty($otp_code)) {
                        $error->add('empty_password', __('<strong>Error</strong>: Password not authenticated', 'actions-pack'));
                    }

                    return $error;
                }

                if ($user) {
                    $email = $user->data->user_email; 
                    $result = $this->authenticate_otp($mobile_phone, $otp_code, $user->ID);
                 
                    if ($result == 'validated') {
                        wsol_sms_to_deletePastdata($mobile_phone, $email, $ip, 'Login');
                        wsol_sms_to_deletePastdata($mobile_phone, $email, $ip, 'Password Validation');
                        return $user;
                    } else {
                        if ($result == 'blocked') {
                            //get the block time for the user
                            $block_time = wsol_sms_to_getBlockTime($wpdb, $user->data->ID);
                            return new \WP_Error(
                                    'invalid_pasword',
                                    __('You have been blocked for ' . $block_time . ' minutes. Please try again later', 'actions-pack')
                            );
                        }

                        if ($result == 'password not authenticated') {
                           $recorded = wsol_sms_to_log_login_register_action($user->ID,  $email, $mobile_phone,  $otp_code, $ip, 'Password Validation' ); 
                            return new \WP_Error(
                                    'invalid_pasword',
                                    __('Password not authenticated', 'actions-pack')
                            );
                        }

                        if ($result == 'password has expired') {
                            return new \WP_Error(
                                    'invalid_pasword',
                                    __('Password has expired. Please enter your mobile phone, click Send to receive a new Password and try to login again', 'actions-pack')
                            );
                        }
                    }
                } else {
                    return new \WP_Error(
                        //'invalid_phone',  __('No registered user with this phone number. Please try again', 'actions-pack'));
                        'invalid_phone',  __('Password not authenticated', 'actions-pack'));
                }
            }
        } else {
            return $user;
        }
    }


    /**
     * Authenticate OTP
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     * @param mobile_phone, mobile_phone number, $user_id
     * @return User
     */
    public function authenticate_otp($mobile_phone, $otp_code, $user_id) {
        global $wpdb, $date_and_Password;


        //get the no of tries for the password
        if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['login_no_of_retries_value'])) {
            $login_no_of_retries = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['login_no_of_retries_value'];
            if (empty($login_no_of_retries)) {
                $login_no_of_retries = 3;
            }
        }

        //get password_validation No of tries
        if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validation_no_of_retries_value'])) {
            $password_validation_no_of_retries = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validation_no_of_retries_value'];
            if (empty($password_validation_no_of_retries)) {
                $password_validation_no_of_retries = 3;
            }
        }

        //get the no of tries for the password
        if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validity_value'])) {
            $password_validity = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validity_value'];
            if (empty($password_validity)) {
                $password_validity = 60;
            }
        }

        $ip = wsol_sms_to_getRealIpAddr();
        $action = 'Login';

        //get the block time for the user
        $block_time = wsol_sms_to_getBlockTime($wpdb, $user_id);

        $blocked = wsol_sms_to_checkIfBlocked($mobile_phone, null, $ip, $action, $login_no_of_retries, $password_validation_no_of_retries, $block_time, $password_validity);

        if ($blocked == true) {
            return 'blocked';
        }

        $date_and_Password = wsol_sms_to_getPasswordandValidity($mobile_phone, null, $ip, $action, $password_validity);
        
        if (((!isset($date_and_Password[0])) || ($date_and_Password[0] == null)) &&
        ((!isset($date_and_Password[1])) || ($date_and_Password[1] == null))) {
         return 'password not authenticated';   
        } 
        
        else
            
            
        if ((isset($date_and_Password[0])) && ($date_and_Password[0] != null)) {
            //password is still valid
            $password = $date_and_Password[1];

            if ($password == $otp_code) {
                return 'validated';
            } else {
                return 'password not authenticated';
            }
        } else {
            return 'password has expired';
        }
        return null;
    }

    /**
     * Style my custom login
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     */    
    public function my_custom_login() {
        
          wp_enqueue_style( 'admin-css', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/custom-login-style.css' );
          

    }

    
    /**
     * Add javascript for login page
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     */
    
    function login_type() {
        wp_enqueue_script('login_enqueue_scripts', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/otp-login-form.js', true, WSOL_WP_SMS_TO_OTP_VERSION);
    }

    /**
     * Add OTP in login form
     * 
     * @author Christodoulou Panikos
     * @email christodoulou.panicos@cytanet.com.cy
     */     
    public function add_otp_in_login_form() {

        include_once WSOL_WP_SMS_TO_OTP_DIR . "includes/templates/otp-login-form.php";
        
    }


    /**
     * @param $mobile_phone_number
     * @param null $user_id
     *
     * @return bool
     */
    private function check_mobile_phone_number($mobile_phone_number, $user_id = null) {
        
        $trimmed_mobile_phone_number = str_replace(' ', '', $mobile_phone_number);
        if ($user_id) {
            $result = $this->db->get_results("SELECT * from `{$this->tb_prefix}usermeta` WHERE meta_key = 'mobile_phone' AND REPLACE(meta_value, ' ', '') = '{$trimmed_mobile_phone_number}' AND user_id != '{$user_id}'");
        } else {
            $result = $this->db->get_results("SELECT * from `{$this->tb_prefix}usermeta` WHERE meta_key = 'mobile_phone' AND REPLACE(meta_value, ' ', '') = '{$trimmed_mobile_phone_number}'");
        }

        if ($result) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param $user_id
     */
    private function delete_user_mobile_phone($user_id) {
        $this->db->delete(
                $this->tb_prefix . "usermeta",
                array(
                    'user_id' => $user_id,
                    'meta_key' => 'mobile_phone',
                )
        );
    }

    /**
     * @param $user_id
     */
    public function check_admin_duplicate_number($user_id) {
        // Get user mobile_phone
        $user_mobile_phone = get_user_meta($user_id, 'mobile_phone', true);
        
        if (empty($user_mobile_phone)) {
            return;
        }

        // Delete user mobile_phone
        if ($this->check_mobile_phone_number($user_mobile_phone, $user_id)) {
            $this->delete_user_mobile_phone($user_id);
        }
    }

    public function add_smsto_mobile_phone_field_to_newuser_form() {
        include_once WSOL_WP_SMS_TO_OTP_DIR . "includes/templates/mobile_phone-field.php";
    }
    
    /**
     * @param $fields
     *
     * @return mixed
     */
    public function add_smsto_mobile_phone_field_to_profile_form($fields) {
        $fields['mobile_phone'] = __('Mobile Number', 'wp-sms-to-otp');

        return $fields;
    }

    public function add_smsto_mobile_phone_field_to_register_form() {
        $mobile_phone = ( isset($_POST['mobile_phone']) ) ? sanitize_text_field($_POST['mobile_phone']) : '';
        include_once WSOL_WP_SMS_TO_OTP_DIR . "includes/templates/mobile_phone-field-register.php";
    }

    /**
     * @param $errors
     * @param $sanitized_user_login
     * @param $user_email
     *
     * @return mixed
     */
    public function registration_errors($errors, $sanitized_user_login, $user_email) {
        $error = false;

        if (empty($_POST['mobile_phone'])) {
            $errors->add('first_name_error', __('<strong>ERROR</strong>: You must include a mobile_phone number.', 'wp-sms-to-otp'));
        }

        if (preg_match('/^[0-9\-\(\)\/\+\s]*$/', sanitize_text_field($_POST['mobile_phone']), $matches) == false) {
            $errors->add('invalid_mobile_phone_number', __('Please enter a valid mobile_phone number', 'wp-sms-to-otp'));
            $error = true;
        }
        if (!$error && !isset($matches[0])) {
            $errors->add('invalid_mobile_phone_number', __('Please enter a valid mobile_phone number', 'wp-sms-to-otp'));
            $error = true;
        }

        if (!$error && isset($matches[0]) && strlen($matches[0]) < 10) {
            $errors->add('invalid_mobile_phone_number', __('Please enter a valid mobile_phone number', 'wp-sms-to-otp'));
            $error = true;
        }

        if (!$error && isset($matches[0]) && strlen($matches[0]) > 14) {
            $errors->add('invalid_mobile_phone_number', __('Please enter a valid mobile_phone number', 'wp-sms-to-otp'));
        }
        
        if ($this->check_mobile_phone_number(sanitize_text_field($_POST['mobile_phone']))) {
            $errors->add('duplicate_mobile_phone_number', __('<strong>ERROR</strong>: This mobile_phone is already registered, please choose another one.', 'wp-sms-to-otp'));
        }

        return $errors;
    }

    /**
     * @param $user_id
     */
    public function save_register($user_id) {
        if (isset($_POST['mobile_phone'])) {
            update_user_meta($user_id, 'mobile_phone', sanitize_text_field($_POST['mobile_phone']));
        }
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.2.0
     */
    public function load_international_input() {

        //Register IntelTelInput Assets
        wp_enqueue_style('wpsmstootp-intel-tel-input', WSOL_WP_SMS_TO_OTP_URL . 'assets/css/intlTelInput.min.css', true, WSOL_WP_SMS_TO_OTP_VERSION);
        wp_enqueue_script('wpsmstootp-intel-tel-input', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/intel/intlTelInput.min.js', array('jquery'), WSOL_WP_SMS_TO_OTP_VERSION, true);
        wp_enqueue_script('wpsmstootp-intel-script', WSOL_WP_SMS_TO_OTP_URL . 'assets/js/intel/intel-script.js', true, WSOL_WP_SMS_TO_OTP_VERSION, true);

        // Localize the IntelTelInput
        $tel_intel_vars = array();
        $only_countries_option = WSOL_SMS_TO_Option::getOption('international_mobile_phone_only_countries');
        $preferred_countries_option = WSOL_SMS_TO_Option::getOption('international_mobile_phone_preferred_countries');

        if ($only_countries_option) {
            $tel_intel_vars['only_countries'] = $only_countries_option;
        } else {
            $tel_intel_vars['only_countries'] = '';
        }

        if ($preferred_countries_option) {
            $tel_intel_vars['preferred_countries'] = $preferred_countries_option;
        } else {
            $tel_intel_vars['preferred_countries'] = '';
        }

        if (WSOL_SMS_TO_Option::getOption('international_mobile_phone_auto_hide')) {
            $tel_intel_vars['auto_hide'] = true;
        } else {
            $tel_intel_vars['auto_hide'] = false;
        }

        if (WSOL_SMS_TO_Option::getOption('international_mobile_phone_national_mode')) {
            $tel_intel_vars['national_mode'] = true;
        } else {
            $tel_intel_vars['national_mode'] = false;
        }

        if (WSOL_SMS_TO_Option::getOption('international_mobile_phone_separate_dial_code')) {
            $tel_intel_vars['separate_dial'] = true;
        } else {
            $tel_intel_vars['separate_dial'] = false;
        }

        $tel_intel_vars['util_js'] = WSOL_WP_SMS_TO_OTP_URL . 'assets/js/intel/utils.js';

        wp_localize_script('wpsmstootp-intel-script', 'sms_to_otp_intel_tel_input', $tel_intel_vars);
    }

}

new WSOL_SMS_TO_Features();
