<?php

namespace WSOL_WP_SMS_TO_OTP;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

/**
 * WSOL_WP_SMS_TO_OTP gateway class
 */
class WSOL_SMS_TO_Gateway {

	public $client_id;
	public $secret;
        public $api_key;
	public $has_key = false;
	public $validateNumber = "";
	public $help = false;
	public $bulk_send = true;
	public $from;
	public $to;
        public $campaign_recipients;        
        public $_id;
	public $msg;
	protected $db;
	protected $tb_prefix;
	public $options;

	public function __construct() {
		global $wpdb;

		$this->db        = $wpdb;
		$this->tb_prefix = $wpdb->prefix;
		$this->options   = WSOL_SMS_TO_Option::getOptions();               

		if ( isset( $this->options['send_unicode'] ) and $this->options['send_unicode'] ) {
			//add_filter( 'sms_to_otp_msg', array( $this, 'applyUnicode' ) );
		}

		// Add Filters
		add_filter( 'sms_to_otp_to', array( $this, 'modify_bulk_send' ) );
	}

	/**
	 * Initial Gateway
	 *
	 * @return mixed
	 */
	public static function initial() {
				// Include gateway
		include_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/gateways/class-wpsmstootp-gateway-wpsmstootp.php';
                $gateway_name = 'wpsmstootp'; 
                $class_name = '\\WSOL_WP_SMS_TO_OTP\\Gateway\\' . $gateway_name;
                $wpsmstootp        = new $class_name();
                        

		// Set client_id and secret
                $wpsmstootp->api_key = WSOL_SMS_TO_Option::getOption( 'gateway_wpsmstootp_api_key' );
                $wpsmstootp->sender_id = WSOL_SMS_TO_Option::getOption( 'gateway_wpsmstootp_sender_id' );
               

		$gateway_key = WSOL_SMS_TO_Option::getOption( 'wpsmstootp_gateway_key' );

		// Set api key
		if ( $wpsmstootp->has_key && $gateway_key ) {
			$wpsmstootp->has_key = $gateway_key;
		}

		// Show gateway help configuration in gateway page
		if ( $wpsmstootp->help ) {
			add_action( 'sms_to_otp_after_gateway', function () {
				echo ' < p class="description" > ' . esc_html($wpsmstootp->help) . '</p > ';
			} );
		}

		// Check unit credit gateway
		if ( $wpsmstootp->unitrial == true ) {
			$wpsmstootp->unit = __( 'Credit', 'wp - sms' );
		} else {
			$wpsmstootp->unit = __( 'SMS', 'wp - sms' );
		}

		// Set sender id
		if ( ! $wpsmstootp->from ) {
			$wpsmstootp->from = WSOL_SMS_TO_Option::getOption( 'gateway_wpsmstootp_sender_id' );
		}

		// Unset gateway key field if not available in the current gateway class.
		add_filter( 'sms_to_otp_gateway_settings', function ( $filter ) {
			global $wpsmstootp;

			if ( ! $wpsmstootp->has_key ) {
				unset( $filter['wpsmstootp_gateway_key'] );
			}

			return $filter;
		} );
           
		// Return gateway object
		return $wpsmstootp;                
	}

	/**
	 * @param $sender
	 * @param $message
	 * @param $to
	 * @param $response
	 * @param string $status
	 *
	 * @return false|int
	 */
	public function log_message($_id,  $sender, $message, $to, $response, $status = 'ONGOING' ) { 
		return $this->db->insert(
			$this->tb_prefix . "smstootp_send",
			array(
                                '_id'      => $_id,
				'date'      => WSOL_WP_SMS_TO_OTP_CURRENT_DATE,
				'sender'    => $sender,
				'message'   => $message,
				'recipient' => implode( ',', $to ),
				'response'  => 'message_id '.var_export( $response, true ),
				'status'    => $status,
			)
		);
	}
        
	/**
         * @author Christodoulou Panikos
         * @email christodoulou.panicos@cytanet.com.cy* 
	 * @param $sender
	 * @param $message
	 * @param $to
	 * @param $response
	 * @param string $status
	 *
	 * @return false|int
	 */
	public function log_campaign($_id, $sender, $message, $to, $response, $status = 'ONGOING' ) {
		return $this->db->insert(
			$this->tb_prefix . "smstootp_send",
			array(
                                '_id'      => $_id,
				'date'      => WSOL_WP_SMS_TO_OTP_CURRENT_DATE,
				'sender'    => $sender,
				'message'   => $message,
				'recipient' => $to,
				'response'  => 'campaign_id '.var_export( $response, true ),
				'status'    => $status,
			)
		);
	}        

	/**
	 * Apply Unicode for non-English characters
	 *
	 * @param string $msg
	 *
	 * @return string
	 */
	public function applyUnicode( $msg = '' ) {
		$encodedMessage = bin2hex( mb_convert_encoding( $msg, 'utf-16', 'utf-8' ) );

		return $encodedMessage;
	}

	/**
	 * @var
	 */
	static $get_response;

	/**
	 * @return mixed|void
	 */
	public static function gateway() {
		$gateways = array(
			''               => array(
				'default' => __( 'Please select your gateway', 'wpsmstootp' ),
			),
			'cyprus'         => array(
				'websmscy' => 'websms.com.cy',
				'smsnetgr' => 'sms.net.gr',
                                'wpsmstootp' => 'sms.to',
			),
		
		);

		return apply_filters( 'wpsmstootp_gateway_list', $gateways );
	}

	/**
	 * @return string
	 */
	public static function status() {
		global $wpsmstootp;

		//Check that, Are we in the Gateway WSOL_WP_SMS_TO_OTP tab setting page or not?
		if ( is_admin() AND isset( $_REQUEST['page'] ) AND isset( $_REQUEST['tab'] ) AND sanitize_text_field($_REQUEST['page']) == 'wp-sms-to-otp-settings' AND sanitize_text_field($_REQUEST['tab'] == 'gateway' )) {

			// Get credit
			$result = $wpsmstootp->GetCredit();

			if ( is_wp_error( $result ) ) {
				// Set error message
				self::$get_response = var_export( $result->get_error_message(), true );

				// Update credit
				update_option( 'wpsmstootp_gateway_credit', 0 );

				// Return html
				return '<div class="wpsmstootp-no-credit"><span class="dashicons dashicons-no"></span> ' . __( 'Deactive!', 'wp-sms-to-otp' ) . '</div>';
			}
			// Update credit
			if ( ! is_object( $result ) ) {
				update_option( 'wpsmstootp_gateway_credit', $result );
			}
			self::$get_response = var_export( $result, true );

			// Return html
			return '<div class="wpsmstootp-has-credit"><span class="dashicons dashicons-yes"></span> ' . __( 'Active!', 'wp-sms-to-otp' ) . '</div>';
		}
	}

	/**
	 * @return mixed
	 */
	public static function response() {
		return self::$get_response;
	}

	/**
	 * @return mixed
	 */
	public static function help() {
		global $wpsmstootp;

		// Get gateway help
		return $wpsmstootp->help;
	}

	/**
	 * @return mixed
	 */
	public static function from() {
		global $wpsmstootp;
		// Get gateway from
		return $wpsmstootp->from;
	}

	/**
	 * @return string
	 */
	public static function bulk_status() {
		global $wpsmstootp;

		// Get bulk status
		if ( $wpsmstootp->bulk_send == true ) {
			// Return html
			return '<div class="wpsmstootp-has-credit"><span class="dashicons dashicons-yes"></span> ' . __( 'Supported', 'wp-sms-to-otp' ) . '</div>';
		} else {
			// Return html
			return '<div class="wpsmstootp-no-credit"><span class="dashicons dashicons-no"></span> ' . __( 'Does not support!', 'wp-sms-to-otp' ) . '</div>';
		}
	}

	/**
	 * @return int
	 */
	public static function credit() {
		global $wpsmstootp;

		// Get credit
		$result = $wpsmstootp->GetCredit();

		if ( is_wp_error( $result ) ) {
			update_option( 'wpsmstootp_gateway_credit', 0 );

			return 0;
		}

		if ( ! is_object( $result ) ) {
			update_option( 'wpsmstootp_gateway_credit', $result );
		}

		return $result;
	}

	/**
	 * Modify destination number
	 *
	 * @param array $to
	 *
	 * @return array/string
	 */
	public function modify_bulk_send( $to ) {
		global $wpsmstootp;
		if ( ! $wpsmstootp->bulk_send ) {
			return array( $to[0] );
		}

		return $to;
	}

}
