<?php

use WSOL_WP_SMS_TO_OTP\WSOL_SMS_TO_Gateway;
use WSOL_WP_SMS_TO_OTP\WSOL_SMS_TO_Option;

include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

/**
 * @return mixed
 */
function wsol_sms_to_otp_initial_gateway() {
    require_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/class-wpsmstootp-option.php';

    return WSOL_SMS_TO_Gateway::initial();
}

add_action('rest_api_init', 'wsol_sms_to_register_route_otp');

add_action('wp_ajax_wsol_sms_to_get_updates_from_db_otp', 'wsol_sms_to_get_updates_from_db_otp');
add_action('wp_ajax_nopriv_wsol_sms_to_get_updates_from_db_otp', 'wsol_sms_to_get_updates_from_db_otp');

add_action('wp_ajax_wsol_sms_to_save_and_send_otp_login', 'wsol_sms_to_save_and_send_otp_login');
add_action('wp_ajax_nopriv_wsol_sms_to_save_and_send_otp_login', 'wsol_sms_to_save_and_send_otp_login');

require_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/class-wpsmstootp-option.php';



/**
 * Register Route
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 */
function wsol_sms_to_register_route_otp() {
    register_rest_route('wp-sms-to-otp', 'get_post', array(
        'methods' => 'POST',
        'callback' => 'wsol_sms_to_update_db_otp',
        'permission_callback' => '__return_true'
    ));
}

if ( is_admin() && ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX ) ) {
   add_action('init', 'wsol_sms_to_update_credit_balance');
}
 

/**
 * Update Credit Balance only if in admin
 * Will run on every page load if it is set to be displayed in admin menu
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 */
function wsol_sms_to_update_credit_balance() {

    $options = \WSOL_WP_SMS_TO_OTP\WSOL_SMS_TO_Option::getOptions();
    if (isset($options['account_credit_in_menu'])) {
        $api_key = isset($options['gateway_wpsmstootp_api_key']) ? $options['gateway_wpsmstootp_api_key'] : '';
        $existing_credit = get_option('wpsmstootp_gateway_credit');
        if (isset($existing_credit) && isset($api_key)) {
            $response = wp_remote_get(\WSOL_WP_SMS_TO_OTP\Gateway\wpsmstootp::getTariff() . '/api/balance?api_key=' . $api_key);
            $body = json_decode(wp_remote_retrieve_body($response));

            if ((200 == wp_remote_retrieve_response_code($response)) && (!is_wp_error($response))) {
                $result = round($body->balance, 2) . ' ' . $body->currency;
                if ($result != $existing_credit) {
                    update_option('wpsmstootp_gateway_credit', $result);
                }
            }
        }
    }
}

/**
 * Update DB
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param WP_REST_Request
 * @return Response
 */
function wsol_sms_to_get_updates_from_db_otp() {

    global $wpdb;

    //get last 15 mins interval
    $time_interval_update = date('Y-m-d H:i:s', current_time('timestamp') - (60 * 15));
    $time_interval_create = date('Y-m-d H:i:s', current_time('timestamp') - (60 * 60 * 24 * 3));

    $get_latest_updated_data = $wpdb->get_results(
            $wpdb->prepare(
                    "SELECT _id, cost, status, type, failed, sms_parts, pending, sent FROM $wpdb->prefix" . "smstootp_send WHERE "
                    . "(updated_at > %s or updated_at is null) and date > %d LIMIT 1000",
                    $time_interval_update, $time_interval_create
            ), ARRAY_A
    );

    $response_init = array();
    $response_init['latest_updated_data'] = json_encode($get_latest_updated_data);
    $response = array_map('sanitize_text_field', $response_init);
    echo $response['latest_updated_data'];
    exit;
}

/**
 * Update DB
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param WP_REST_Request
 * @return Response
 */
function wsol_sms_to_update_db_otp(WP_REST_Request $request) {

    global $wpdb;
    $table = $wpdb->prefix . 'smstootp_send';
    //parameters is an array
    $parameters = $request->get_params();

    if ((isset($parameters['status'])) &&
            (isset($parameters['messageId'])) &&
            (isset($parameters['phone'])) &&
            (isset($parameters['trackingId'])) &&
            (isset($parameters['price']))) {

        $status = $parameters['status'];
        $messageId = $parameters['messageId'];
        $trackingId = $parameters['trackingId'];
        $price = $parameters['price'];

        $updated_at = WSOL_WP_SMS_TO_OTP_CURRENT_DATE;

        $wpdb->query("UPDATE $table SET status='$status', updated_at='$updated_at', cost='$price', type='sms' WHERE response Like '%$messageId%' and response Like '%message_id%'");
        $wpdb->query("UPDATE $table SET status='$status', 'updated_at='$updated_at'  WHERE response Like '%$trackingId%' and response Like '%campaign_id%'");        
        
    }
}

/**
 * Save OTP in DB and Send it to Wordpress User
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param api_key, mobile_phone
 * @return Response
 */
function wsol_sms_to_save_and_send_otp_login() {
    
    require_once WSOL_WP_SMS_TO_OTP_DIR . 'includes/class-wpsmstootp-option.php';
    global $wpsmstootp, $wpdb, $login_no_of_retries;

    //get the api key
    if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['gateway_wpsmstootp_api_key'])) {
        $api_key = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['gateway_wpsmstootp_api_key'];
    }

    //get login No of tries
    if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['login_no_of_retries_value'])) {
        $login_no_of_retries = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['login_no_of_retries_value'];
        if (empty($login_no_of_retries)) {
            $login_no_of_retries = 3;
        }
    }


    //get password_validation No of tries
    if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validation_no_of_retries_value'])) {
        $password_validation_no_of_retries = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validation_no_of_retries_value'];
        if (empty($password_validation_no_of_retries)) {
            $password_validation_no_of_retries = 3;
        }
    }

    //get the validity time of a password
    if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validity_value'])) {
        $password_validity = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['password_validity_value'];
        if (empty($password_validity)) {
            $password_validity = 60;
        }
    }

    //get the user by phone
    $mobile_phone_posted = isset($_POST['mobile_phone']) ? trim(sanitize_text_field($_POST['mobile_phone'])) : "";
    $mobile_phone = str_replace(' ', '', $mobile_phone_posted);
    if (($mobile_phone)) {
        $user = wsol_sms_to_get_user_by_mobile_phone_otp('mobile_phone', $mobile_phone);
        if ((!isset($user)) || ($user == null)) {
            $response_init = array();
            $response_init['message'] = "otp sent";
            $response = array_map( 'sanitize_text_field', $response_init );
            echo json_encode($response);
            exit;
        } else {
            $user_id = $user->data->ID;
            $email = $user->data->user_email;
        }
    } else {
            $response_init = array();
            $response_init['message'] = "otp sent";
            $response = array_map( 'sanitize_text_field', $response_init );
            echo json_encode($response);
        exit;
    }


    $ip = wsol_sms_to_getRealIpAddr();
    $action = 'Login';


    //get the block time for the user
    $block_time = wsol_sms_to_getBlockTime($wpdb, $user_id);

    $blocked = wsol_sms_to_checkIfBlocked($mobile_phone, $email, $ip, $action, $login_no_of_retries, $password_validation_no_of_retries, $block_time, $password_validity);
    if ($blocked == true) {
        
        $response_init = array();
        $response_init['message'] = "blocked";
        $response_init['time'] = $block_time;
        $response = array_map( 'sanitize_text_field', $response_init );        
        echo json_encode($response);
        exit;
    }

    $date_and_Password = wsol_sms_to_getPasswordandValidity($mobile_phone, $email, $ip, $action, $password_validity);
    if ((isset($date_and_Password[0])) && ($date_and_Password[0] != null)) {
        //password is still valid
        $password = $date_and_Password[1];
            $response_init = array();
            $response_init['message'] = "still valid password";
            $response_init['password_validity'] = $password_validity;
            $response = array_map( 'sanitize_text_field', $response_init );
            echo json_encode($response);
        exit;
    } else {
        //we will send sms 
        $password = rand(1000, 9999);
        $estimated_single_message_cost = wsol_sms_to_estimateSingleMessage($mobile_phone, $password, $api_key);
        $balance = wsol_sms_to_getBalance($api_key);
        if (($estimated_single_message_cost != null) && ($balance != null)) {
            if ($estimated_single_message_cost > $balance) {
                $response_init = array();
                $response_init['message'] = "insufficient balance";
                $response = array_map( 'sanitize_text_field', $response_init );     
                echo json_encode($response);
                exit;
            } else {

                $sender_id = WSOL_SMS_TO_Option::getOption('gateway_wpsmstootp_sender_id');
                if ((!isset($sender_id)) || (empty($sender_id)) || ($sender_id == null)) {
                    $wpsmstootp->from = 'SMSto';
                } else {
                    $wpsmstootp->from = $sender_id;
                }
                $wpsmstootp->to = array_map( 'sanitize_text_field', explode(",", $mobile_phone) );                                
                $message = $password . ' is your one time password to login';
                $wpsmstootp->msg = $message;
                $sms_response = $wpsmstootp->SendSMS();
                
                    if ((200 == wp_remote_retrieve_response_code($sms_response)) && (!is_wp_error($sms_response))) {
                    $recorded = wsol_sms_to_log_login_register_action($user_id, $email, $mobile_phone, $password, $ip, $action);
                    $response_init = array();
                    $response_init['message'] = "otp sent";
                    $response = array_map('sanitize_text_field', $response_init);
                    echo json_encode($response);
                    exit;
                } else {
                    $response_init = array();
                    $response_init['message'] = "otp sent";
                    $response = array_map('sanitize_text_field', $response_init);
                    echo json_encode($response);
                    exit;
                }
            }
        } else {
            $response_init = array();
            $response_init['message'] = "otp sent";
            $response = array_map('sanitize_text_field', $response_init);
            echo json_encode($response);
            exit;
        }
    }
}

/**
 * Retrieve User mobile phone from wp_usermeta 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param mobile_phone, mobile_phone number
 * @return User
 */
function wsol_sms_to_get_user_by_mobile_phone_otp($db_field, $value) {
    global $wpdb;

    $user_id = $wpdb->get_row(
            $wpdb->prepare(
                    "SELECT user_id FROM $wpdb->prefix" . "usermeta WHERE meta_key = %s
				 AND REPLACE(meta_value, ' ', '') = %d LIMIT 1",
                    $db_field, $value
            )
    );

    if ($user_id) {
        $array = json_decode(json_encode($user_id), true);
        $user = get_user_by('ID', $array ["user_id"]);
        return $user;
    } else {
        return null;
    }
}

/**
 * Get the block time for a user 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param user id
 * @return integer
 */
function wsol_sms_to_getBlockTime($wpdb, $user_id) {

    if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['block_time_enable'])) {
        $block_time_enable = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['block_time_enable'];
    } else {
        $block_time_enable = '0';
    }

    //if block time isset
    if ($block_time_enable == '1') {

        if (isset(WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['block_time_content'])) {
            $roleAndTimes = WSOL_SMS_TO_Option::getOptions($pro = true, $setting_name = 'wpsmstootp_settings')['block_time_content'];
        }

        //we use sum in case there are more than one entries for block time for same role
        $sum_blocktime = 0;
        $blocktime = 0;

        foreach ($roleAndTimes as $roleAndTime) {
            if (!empty($roleAndTime['user_role'])) {
                $user_role = $roleAndTime['user_role'];
                $no_roles = $wpdb->get_var("SELECT COUNT(*) FROM `{$wpdb->prefix}usermeta` where `meta_key` = 'wp_capabilities' and `user_id` = '$user_id' and `meta_value` LIKE '%$user_role%'");

                $no_roles = (int) $no_roles;
                if ($no_roles > 0) {
                    $blocktime = $roleAndTime['block_time'];
                    $sum_blocktime = $sum_blocktime + $blocktime;
                }
            }
        }
        if (isset($sum_blocktime)) {
            return $sum_blocktime;
        } else {
            return 0;
        }
    } else {
        return 0;
    }
}

/**
 * Get OTP cost estimation 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param mobile_phone, text=4digits, api_key
 * @return cost
 */
function wsol_sms_to_estimateSingleMessage($mobile_phone, $text, $api_key) {


    $bodyContent = array(
        'sender_id' => 'SENDER',
        'to' => $mobile_phone,
        'message' => $text,
    );

    if ($bodyContent) {
        $body = json_encode($bodyContent);
    }


    $url = \WSOL_WP_SMS_TO_OTP\Gateway\wpsmstootp::getWsdl_link() . '/sms/estimate';

    $headers = array(
        'Authorization' => 'Bearer ' . $api_key,
        'Content-Type' => 'application/json',
        'Accept' => 'application/json'
    );

    $args = array(
        'body' => $body,
        'timeout' => '15',
        'redirection' => '10',
        'httpversion' => '1.0',
        'blocking' => true,
        'headers' => $headers,
    );


    $response = wp_remote_post($url, $args);



    if (!is_wp_error($response) && 200 == wp_remote_retrieve_response_code($response)) {

        $body = json_decode(wp_remote_retrieve_body($response));
        return ($body->estimated_cost);
    } else {
        return null;
    }
}

/**
 * Get User balance 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param api_key
 * @return balance
 */
function wsol_sms_to_getBalance($api_key) {
    
    $url = \WSOL_WP_SMS_TO_OTP\Gateway\wpsmstootp::getTariff() . '/api/balance';

    $headers = array(
        'Authorization' => 'Bearer ' . $api_key
    );

    $args = array(
        'timeout' => '15',
        'redirection' => '10',
        'httpversion' => '1.0',
        'blocking' => true,
        'headers' => $headers,
    );

    $response = wp_remote_get($url, $args);

    if (!is_wp_error($response) && 200 == wp_remote_retrieve_response_code($response)) {
        $body = json_decode(wp_remote_retrieve_body($response));
        return ($body->balance);
    } else {
        return null;
    }    
}

function wsol_sms_to_getRealIpAddr() {
    if (!empty($_SERVER['HTTP_CLIENT_IP'])) {   //check ip from share internet
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {   //to check ip is pass from proxy
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    } else {
        $ip = $_SERVER['REMOTE_ADDR'];
    }
    return $ip;
}

/**
 * Log Login and Register Actions 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param user_id, telephone, email, ip, password, 
 * @return boolean
 */
function wsol_sms_to_log_login_register_action($user_id, $email, $telephone, $password, $ip, $action) {

    global $wpdb;
    return $wpdb->insert(
                    $wpdb->prefix . "smstootp_login_register_actions",
                    array(
                        'user_id' => $user_id,
                        'date' => WSOL_WP_SMS_TO_OTP_CURRENT_DATE,
                        'email' => $email,
                        'telephone' => $telephone,
                        'password' => $password,
                        'ip' => $ip,
                        'action' => $action
                    )
    );
}

/**
 * Check if user is blocked 
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param mobile_phone, email, ip, login tries, password tries, block time, password validity
 * @return boolean
 */
function wsol_sms_to_checkIfBlocked($mobile_phone, $email, $ip, $action, $login_no_of_retries, $password_validation_no_of_retries, $block_time, $password_validity) {
    global $wpdb;
    $login_blocked;

    //to check that the user is not blocked we check login tries and password validation tries 
    //For Login
    $get_rows_for_login_tries = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = '$action' AND (`telephone` = '$mobile_phone' OR `email` = '$email' OR `ip` = '$ip')", ARRAY_A);

    if ((count($get_rows_for_login_tries) < $login_no_of_retries)) {
        $login_blocked = false;
    } else {
        //for login
        $get_login_max_date = $wpdb->get_results("SELECT date as loginmaxdate FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = '$action' AND (`telephone` = '$mobile_phone' OR `email` = '$email' OR `ip` = '$ip') ORDER BY id DESC LIMIT 1", ARRAY_A);
        $login_max_date = json_encode($get_login_max_date[0]['loginmaxdate']);
        $login_max_date = str_replace('"', '', $login_max_date);

        $now_date_as_date_object = new DateTime();

        $login_max_date_as_date_object_plus_password_validity = new DateTime($login_max_date);
        $login_max_date_as_date_object_plus_password_validity->add(new DateInterval('PT' . $password_validity . 'S'));

        //handle password validity period before check for blocked
        if ($login_max_date_as_date_object_plus_password_validity > $now_date_as_date_object) {
            $login_blocked = false;
        } else {

            //add the block time to the max date but also the validity we subtracted earlier
            $login_max_date_as_date_object = new DateTime($login_max_date);
            $login_max_date_as_date_object->add(new DateInterval('PT' . $block_time . 'M'));
            $login_max_date_as_date_object->add(new DateInterval('PT' . $password_validity . 'S'));

            if ($now_date_as_date_object < $login_max_date_as_date_object) {
                $login_blocked = true;
            } else {
                wsol_sms_to_deletePastdata($mobile_phone, $email, $ip, 'Login');
                $login_blocked = false;
            }
        }
    }

    $password_validation_blocked;

    $get_rows_for_password_validation_tries = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = 'Password Validation' AND (`telephone` = '$mobile_phone' OR `email` = '$email' OR `ip` = '$ip')", ARRAY_A);

    if (count($get_rows_for_password_validation_tries) < $password_validation_no_of_retries) {
        $password_validation_blocked = false;
    } else {

        //for password validation
        $get_password_validation_max_date = $wpdb->get_results("SELECT date as passwordvalidationmaxdate FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = 'Password Validation' AND (`telephone` = '$mobile_phone' OR `email` = '$email' OR `ip` = '$ip') ORDER BY id DESC LIMIT 1", ARRAY_A);
        $password_validation_max_date = json_encode($get_password_validation_max_date[0]['passwordvalidationmaxdate']);
        $password_validation_max_date = str_replace('"', '', $password_validation_max_date);

        $now_date_as_date_object = new DateTime();

        //add the block time to the max date but also the validity we subtracted earlier
        $password_validation_max_date_as_date_object = new DateTime($password_validation_max_date);

        $password_validation_max_date_as_date_object->add(new DateInterval('PT' . $block_time . 'M'));

        if ($now_date_as_date_object < $password_validation_max_date_as_date_object) {
            $password_validation_blocked = true;
        } else {
            wsol_sms_to_deletePastdata($mobile_phone, $email, $ip, 'Password Validation');
            $password_validation_blocked = false;
        }
    }


    //if both Login and Password validation are false then the user is not blocked. it will return false, otherwise true.
    if (($login_blocked == false) && ($password_validation_blocked == false)) {
        return false;
    } else {
        return true;
    }
}

/**
 * get Password Validity
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 */
function wsol_sms_to_getPasswordandValidity($mobile_phone, $email, $ip, $action, $password_validity) {

    global $wpdb;


    $date_and_Password = array();

    $get_rows = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = '$action' AND (`telephone` = '$mobile_phone' OR `email` = '$email')", ARRAY_A);
    if (count($get_rows) > 0) {

        $get_id_max_password = $wpdb->get_results("SELECT date, password FROM `{$wpdb->prefix}smstootp_login_register_actions` WHERE `action` = '$action' AND (`telephone` = '$mobile_phone' OR `email` = '$email' OR `ip` = '$ip') ORDER BY id DESC LIMIT 1", ARRAY_A);
        $max_date = json_encode($get_id_max_password[0]['date']);
        $max_date = str_replace('"', '', $max_date);

        $password = json_encode($get_id_max_password[0]['password']);
        $password = str_replace('"', '', $password);

        $now_date_as_date_object = new DateTime();
        $max_date_as_date_object = new DateTime($max_date);
        $max_date_as_date_object->add(new DateInterval('PT' . $password_validity . 'S'));

        if ($now_date_as_date_object < $max_date_as_date_object) {
            $date_and_Password [0] = $max_date;
            $date_and_Password [1] = $password;
            return $date_and_Password;
        } else {
            $date_and_Password [0] = null;
            $date_and_Password [1] = null;
            return $date_and_Password;
        }
    } else {
        $date_and_Password [0] = null;
        $date_and_Password [1] = null;
        return $date_and_Password;
    }
}

/**
 * Delete data from db
 * 
 * @author Christodoulou Panikos
 * @email christodoulou.panicos@cytanet.com.cy
 * @param mobile_phone, email, ip, action
 * @return boolean
 */
function wsol_sms_to_deletePastdata($mobile_phone, $email, $ip, $action) {
    global $wpdb;

    $wpdb->query(
            $wpdb->prepare(
                    "DELETE a FROM {$wpdb->prefix}smstootp_login_register_actions a
          WHERE a.action=%s AND (a.telephone=%s OR a.email=%s OR a.ip=%s)",
                    $action, $mobile_phone, $email, $ip
            )
    );
}
